let formatter = ref Format.std_formatter
let columns = ref 72

let print_stats = ref false
let pretty_print = ref false

let repetitions = ref 1

let verbose1 = ref false
let verbose2 = ref false
let all_states = ref false
let show_induction = ref false
let allow_leaks = ref false
let forget_fields = ref false
let value_abstraction = ref "@zero"

let prover_num = ref 1

let is_value_field s = (s = "val") 
let is_lock_field s = (s = "lk" || s = "H_lock" || s = "T_lock")

let inductive_preds = ref true

(* default component tags *)
let list_data_tag = "hd"
let list_link_tag = "tl"
let tree_data_tag = "d"
let tree_link_tags = "l", "r"
let dl_Llink_tag,dl_Rlink_tag = tree_link_tags (* the parser assumes this *)

(* command line arguments *)
let speclist = Arg.align
  [("-repetitions", Arg.Int (fun n -> repetitions := n), "n Verify a number of times repeatedly");
   ("-columns", Arg.Int (fun n -> columns := n), "n Format output for a number of columns");
   ("-verbose", Arg.Unit (fun () ->
                          verbose1 := true;
                          verbose2 := false;
			  all_states := false;
			  show_induction := false), " Display additional internal information");
   ("-allow_leaks", Arg.Set allow_leaks, " Only print a warning in case of memory leaks");
   ("-pretty", Arg.Set pretty_print, " Make printed assertions look simpler");
   ("-stats", Arg.Set print_stats, " Print time and memory statistics");
   ("-all_states", Arg.Set all_states, " Display intermediate states");
   ("-value", Arg.Int (fun n -> value_abstraction := 
	if n = 0 then "@zero"
	else if n = 1 then "@set"
	else if n = 2 then "@list"
	else "@error"), " How to abstract the values of stored in list segments");
   ("-show_induction", Arg.Set show_induction, " Indicate when induction is used during verification");
   ("-very_verbose", Arg.Unit (fun () ->
                            verbose1 := true;
                            verbose2 := true;
			    all_states := true;
                            show_induction := true), " Display more additional internal information");
   ("-prover", Arg.Int (fun n -> prover_num := n), "n Which prover to use (n = 1..2)");
   ("-forget_fields", Arg.Set forget_fields, " Enable forget_fields optimisation")]

let usage_msg =
  "Usage: " ^ Sys.argv.(0) ^ " [options] <files>"
