(** Handling of user-defined inductive predicates.*)
open Exp
open Symbtree

exception Indpred_error of string * Location.t

(** Internal representation of inductive predicates *)
type ip_decl =
    {ip_id: string;
     ip_args: Misc.ident list;
     ip_body: (can_pure * can_form) list;
     ip_node: bool;
     ip_loc: Location.t}

(** Raises [Indpred_error] if an error occurs *)
val add_ip_decl : ip_decl -> unit

(** Raises [Indpred_error] if an error occurs *)
val check_ip_uses : unit -> unit

val instance_ip_lhs : string * can_exp list -> (can_pure * can_form) list
val instance_ip_rhs : string * can_exp list -> (can_pure * can_form) list
val init_ip_env : unit -> unit
val print_ip_env :  Format.formatter -> unit

val add_ip_use : string * int * Location.t -> unit

(** unroll a node declaration in the lhs *) 
val unroll_node_lhs: string -> can_exp -> can_exp * can_exp * string * fields * can_pure

(** unroll a node declaration in the rhs *)
val unroll_node_rhs: string -> can_exp -> can_exp * can_exp * string * fields * can_pure
