(** Abstract syntax tree produced by parsing. *)
type a_component = string

type p_type = string

type p_expression =
	{ pexp_desc: p_expression_desc;
	  pexp_loc: Location.t }

and p_expression_desc =
  | Pexp_ident of string
  | Pexp_num of int
  | Pexp_bool of bool
  | Pexp_prefix of string * p_expression
  | Pexp_infix of string * p_expression * p_expression
  | Pexp_cast of p_expression * p_type       (* impure *)
  | Pexp_fld of p_expression * a_component   (* impure *)
  | Pexp_new of p_type                       (* impure *)
  | Pexp_fun of string * p_expression list
  | Pexp_fcall of string * actual_params     (* impure *)

and actual_params = (string * Location.t) list * p_expression list

and a_expression = p_expression (* without the impure cases *)

type dlink_kind = DL | XL

type a_space_pred =
(*  | Aspred_list of a_component * a_expression * Location.t
  | Aspred_listseg of a_component * string * a_expression * a_expression * a_expression * a_expression list * Location.t
  | Aspred_listsegi of a_component * string * a_expression * a_expression * a_expression * a_expression list * Location.t
  | Aspred_dlseg of dlink_kind * a_component * a_expression * a_expression * a_component * a_expression * a_expression
  | Aspred_tree of a_component * a_component * a_expression
*)
  | Aspred_node of string * a_expression * (a_component * a_expression) list
  | Aspred_indpred of string * (string * Location.t) list * a_expression list * Location.t

type a_proposition =
  | Aprop_exp of a_expression
  | Aprop_spred of a_space_pred
  | Aprop_ifthenelse of a_expression * a_proposition * a_proposition
  | Aprop_star of a_proposition * a_proposition
  | Aprop_barbar of a_proposition * a_proposition
  | Aprop_wande of a_proposition * a_proposition
  | Aprop_box of string * a_proposition

type a_invariant = a_proposition option

type p_statement =
	{ pstm_desc: p_statement_desc;
	  pstm_loc: Location.t }

and p_statement_desc =
  | Pstm_exp of p_expression
  | Pstm_assign of string * p_expression
  | Pstm_fldassign of p_expression * a_component * p_expression
  | Pstm_dispose of p_expression
  | Pstm_block of p_statement list
  | Pstm_if of p_expression option * p_statement * p_statement
  | Pstm_while of a_invariant * p_expression option * p_statement
  | Pstm_withres of string * p_expression * p_statement * string * p_expression list
  | Pstm_action of p_statement * string * string * p_expression list
  | Pstm_parblock of (string option * string * actual_params) list
  | Pstm_return of p_expression option
  | Pstm_break
  | Pstm_continue

type p_action = string * (string * Location.t) list * a_proposition * a_proposition * a_proposition * Location.t

type p_vars = (string * p_type * Location.t) list

type p_item =
  | Pdec_class of string * p_vars * Location.t
  | Pdec_var of string * p_type * Location.t
  | Pdec_fun of string * p_type * (p_vars * p_vars) * (a_invariant * a_invariant) * (p_vars * p_statement list) * Location.t
  | Pdec_resource of string * (string * Location.t) list * a_invariant
	* (p_vars * p_statement list) * p_action list * Location.t
  | Pdec_indpred of string * (string * Location.t) list * a_proposition * Location.t

type p_program = p_item list

(** [iter f s] calls [f] on each statement in [s] *)
val iter : (Location.t -> p_statement_desc -> unit) -> p_statement list -> unit

val a_prop_empty: a_proposition
